/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 1998-2018                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef INFOWIDGET_H
#define INFOWIDGET_H

#include <QLayout>
#include <QSplitter>
#include <QTextBrowser>
#include <QPushButton>
#include <QTabWidget>
#include "TabInterface.h"

namespace cubegui
{
class HtmlWidget;

/**
 * @brief The InfoWidget class
 * The InfoWidget class displays a textual message in a tabbed widget.
 */
class InfoWidget : public QWidget, public TabInterface
{
    Q_OBJECT
public:
    enum ContentType { Info, TreeInfo, PluginInfo };

    static InfoWidget*
    getInstance();

    ~InfoWidget();

    void
    showTreeInfo( const QString& metric,
                  const QString& call,
                  const QString& metricUrl,
                  const QString& callUrl );

    void
    showPluginInfo( const QString& txt,
                    QPushButton*   button );
    void
    activateTab( TreeType type );

    // TabInterface interface
    QWidget*
    widget();
    QString
    label() const;
    QIcon
    icon() const;

public slots:
    void
    activate();

private:
    InfoWidget();

    // tree info components
    QSplitter*    horizontal;
    QSplitter*    vertical;
    QTextBrowser* left;
    QTextBrowser* right;
    QTabWidget*   doc;

    QWidget*    treeInfo;
    ContentType content;

    static InfoWidget* single;
};
}
#endif // INFOWIDGET_H
